@@ -9,9 +9,5 @@ def main():
         '-o',
         help='Where to save the plot, defaults to pipeline.png',
         default='pipeline.png')
-    parser.add_argument('--log',
-                        help='Enables logging to stdout at the '
-                        'specified level',
-                        default=None)
     dag, args = _custom_command(parser)
     dag.plot(output=args.output)
