@@ -9,9 +9,5 @@ def main():
         '-o',
         help='Where to save the report, defaults to pipeline.html',
         default='pipeline.html')
-    parser.add_argument('--log',
-                        help='Enables logging to stdout at the '
-                        'specified level',
-                        default=None)
     dag, args = _custom_command(parser)
     dag.to_markup(path=args.output)
